@@ -53,13 +53,13 @@ struct browse_ctx;
 struct bio_source;
 
 struct browse_ctx *browse_host_dl_create(
-	gpointer owner, gnet_host_t *host, gnet_search_t sh);
+	void *owner, gnet_host_t *host, gnet_search_t sh);
 void browse_host_dl_free(struct browse_ctx **ptr);
-gboolean browse_host_dl_for_search(struct browse_ctx *bc, gnet_search_t sh);
+bool browse_host_dl_for_search(struct browse_ctx *bc, gnet_search_t sh);
 void browse_host_dl_write(struct browse_ctx *bc, char *data, size_t len);
-gboolean browse_host_dl_receive(
+bool browse_host_dl_receive(
 	struct browse_ctx *bc, gnet_host_t *host, struct wrap_io *wio,
-	const char *vendor, guint32 flags);
+	const char *vendor, uint32 flags);
 struct bio_source *browse_host_io_source(struct browse_ctx *bc);
 void browse_host_dl_close(struct browse_ctx *bc);
 void browse_host_dl_search_closed(struct browse_ctx *bc, gnet_search_t sh);
