@@ -970,8 +970,8 @@ dmesh_raw_add(const struct sha1 *sha1, const dmesh_urlinfo_t *info,
 		goto rejected;
 	}
 
-	if (hostiles_check(addr)) {
-		reason = "hostile address";
+	if (hostiles_is_bad(addr)) {
+		reason = "bad hostile address";
 		goto rejected;
 	}
 
