@@ -57,14 +57,14 @@ void dq_close(void);
 void dq_launch_net(struct gnutella_node *n,
 	struct query_hashvec *qhv, unsigned media_mask);
 void dq_node_removed(const struct nid *node_id);
-gboolean dq_got_results(const struct guid *muid, guint count, guint32 status);
-gboolean dq_oob_results_ind(const struct guid *muid, int count);
-void dq_oob_results_got(const struct guid *muid, guint count);
+bool dq_got_results(const struct guid *muid, uint count, uint32 status);
+bool dq_oob_results_ind(const struct guid *muid, int count);
+void dq_oob_results_got(const struct guid *muid, uint count);
 void dq_got_query_status(const struct guid *muid, const struct nid *node_id,
-		guint16 kept);
+		uint16 kept);
 void dq_launch_local(gnet_search_t handle, pmsg_t *mb, query_hashvec_t *qhv);
 void dq_search_closed(gnet_search_t handle);
-gboolean dq_get_results_wanted(const struct guid *muid, guint32 *wanted);
+bool dq_get_results_wanted(const struct guid *muid, uint32 *wanted);
 
 #endif	/* _core_dq_h_ */
 
