@@ -96,6 +96,7 @@ typedef enum {
  */
 
 void gmsg_init(void);
+void gmsg_close(void);
 const char *gmsg_name(uint function);
 gmsg_valid_t gmsg_size_valid(const void *msg, uint16 *size);
 
@@ -143,7 +144,7 @@ void gmsg_install_presend(pmsg_t *mb);
 
 void gmsg_log_bad(const struct gnutella_node *n,
 	const char *reason, ...) G_GNUC_PRINTF(2, 3);
-void gmsg_log_dropped_pmsg(pmsg_t *msg,
+void gmsg_log_dropped_pmsg(const pmsg_t *msg,
 	const char *reason, ...) G_GNUC_PRINTF(2, 3);
 void gmsg_log_split_dropped(
 	const void *head, const void *data, size_t data_len,
