@@ -206,8 +206,7 @@ add_persistent_cache_entry(const char *filename, filesize_t size,
 		 */
 
 		if (fstat(fileno(f), &sb)) {
-			g_warning("add_persistent_cache_entry: could not stat \"%s\"",
-				pathname);
+			g_warning("%s(): could not stat \"%s\": %m", G_STRFUNC, pathname);
 		} else {
 			if (0 == sb.st_size) {
 				fputs(sha1_persistent_cache_file_header, f);
@@ -216,8 +215,7 @@ add_persistent_cache_entry(const char *filename, filesize_t size,
 		}
 		fclose(f);
 	} else {
-		g_warning("add_persistent_cache_entry: could not open \"%s\"",
-			pathname);
+		g_warning("%s(): could not open \"%s\": %m", G_STRFUNC, pathname);
 	}
 	HFREE_NULL(pathname);
 }
@@ -475,17 +473,17 @@ huge_update_hashes(shared_file_t *sf,
 			shared_file_size(sf), shared_file_modification_time(sf),
 			sha1, tth);
 	}
-	if (NULL == tth) {
-		request_tigertree(sf, FALSE);
-	}
 	return TRUE;
 }
 
 /**
- * Get the next file waiting for its hash to be computed from the queue
- * (actually a stack).
+ * Look whether we still need to compute the SHA1 of the given shared file
+ * by looking into our in-core cache to see whether the entry we have is
+ * up-to-date.
  *
- * @return this file.
+ * @param sf	the shared file for which we want to compute the SHA1
+ *
+ * @return TRUE if the file need SHA1 recomputation.
  */
 static bool
 huge_need_sha1(shared_file_t *sf)
@@ -498,27 +496,13 @@ huge_need_sha1(shared_file_t *sf)
 	 * After a rescan, there might be files in the queue which are
 	 * no longer shared.
 	 */
+
 	if (!(SHARE_F_INDEXED & shared_file_flags(sf)))
 		return FALSE;
 
-	/*
-	 * XXX HACK ALERT
-	 *
-	 * We need to be careful here, because each time the library is rescanned,
-	 * we add file to the list of SHA1 to recompute if we don't have them
-	 * yet.  This means that when we rescan the library during a computation,
-	 * we'll add duplicates to our working queue.
-	 *
-	 * Fortunately, we can probe our in-core cache to see if what we have
-	 * is already up-to-date.
-	 *
-	 * XXX It would be best to maintain a hash table of all the filenames
-	 * XXX in our workqueue and not enqueue the work in the first place.
-	 * XXX		--RAM, 21/05/2002
-	 */
-
 	cached = hikset_lookup(sha1_cache, shared_file_path(sf));
-	if (cached) {
+
+	if (cached != NULL) {
 		filestat_t sb;
 
 		if (-1 == stat(shared_file_path(sf), &sb)) {
@@ -530,7 +514,7 @@ huge_need_sha1(shared_file_t *sf)
 			cached->size + (fileoffset_t) 0 == sb.st_size + (filesize_t) 0 &&
 			cached->mtime == sb.st_mtime
 		) {
-			if (GNET_PROPERTY(dbg) > 1) {
+			if (GNET_PROPERTY(share_debug) > 1) {
 				g_warning("ignoring duplicate SHA1 work for \"%s\"",
 					shared_file_path(sf));
 			}
@@ -560,12 +544,15 @@ huge_verify_callback(const struct verify *ctx, enum verify_status status,
 	shared_file_check(sf);
 	switch (status) {
 	case VERIFY_START:
+		if (!huge_need_sha1(sf))
+			return FALSE;
 		gnet_prop_set_boolean_val(PROP_SHA1_REBUILDING, TRUE);
-		return huge_need_sha1(sf);
+		return TRUE;
 	case VERIFY_PROGRESS:
 		return 0 != (SHARE_F_INDEXED & shared_file_flags(sf));
 	case VERIFY_DONE:
 		huge_update_hashes(sf, verify_sha1_digest(ctx), NULL);
+		request_tigertree(sf, TRUE);
 		/* FALL THROUGH */
 	case VERIFY_ERROR:
 	case VERIFY_SHUTDOWN:
@@ -580,8 +567,10 @@ huge_verify_callback(const struct verify *ctx, enum verify_status status,
 }
 
 /**
- * Put the shared file on the stack of the things to do. Activate the timer if
- * this wasn't done already.
+ * Put the shared file on the stack of the things to do.
+ *
+ * We first begin with the computation of the SHA1, and when completed we
+ * will continue with the TTH computation.
  */
 static void
 queue_shared_file_for_sha1_computation(shared_file_t *sf)
@@ -593,9 +582,9 @@ queue_shared_file_for_sha1_computation(shared_file_t *sf)
 	inserted = verify_sha1_enqueue(FALSE, shared_file_path(sf),
 					shared_file_size(sf), huge_verify_callback,
 					shared_file_ref(sf));
-	if (!inserted) {
+
+	if (!inserted)
 		shared_file_unref(&sf);
-	}
 }
 
 /**
@@ -635,15 +624,16 @@ request_sha1(shared_file_t *sf)
 	shared_file_check(sf);
 
 	cached = hikset_lookup(sha1_cache, shared_file_path(sf));
+
 	if (cached && cached_entry_up_to_date(cached, sf)) {
 		cache_dirty = TRUE;
 		cached->shared = TRUE;
 		shared_file_set_sha1(sf, cached->sha1);
 		shared_file_set_tth(sf, cached->tth);
-		request_tigertree(sf, FALSE);
+		request_tigertree(sf, NULL == cached->tth);
 	} else {
 
-		if (GNET_PROPERTY(dbg) > 4) {
+		if (GNET_PROPERTY(share_debug) > 1) {
 			if (cached)
 				g_debug("cached SHA1 entry for \"%s\" outdated: "
 					"had mtime %lu, now %lu",
@@ -717,7 +707,7 @@ huge_sha1_extract32(const char *buf, size_t len, struct sha1 *sha1,
 	 */
 
 	if (huge_improbable_sha1(sha1->data, sizeof sha1->data)) {
-		if (GNET_PROPERTY(dbg)) {
+		if (GNET_PROPERTY(share_debug)) {
 			if (is_printable(buf, len)) {
 				g_warning("%s has improbable SHA1 (len=%lu): %.*s, hex: %s",
 					gmsg_node_infostr(n),
@@ -733,7 +723,7 @@ huge_sha1_extract32(const char *buf, size_t len, struct sha1 *sha1,
 	return TRUE;
 
 bad:
-	if (GNET_PROPERTY(dbg)) {
+	if (GNET_PROPERTY(share_debug)) {
 		if (is_printable(buf, len)) {
 			g_warning("%s has bad SHA1 (len=%u): %.*s",
 				gmsg_node_infostr(n),
@@ -764,7 +754,7 @@ huge_tth_extract32(const char *buf, size_t len, struct tth *tth,
 	return TRUE;
 
 bad:
-	if (GNET_PROPERTY(dbg)) {
+	if (GNET_PROPERTY(share_debug)) {
 		if (is_printable(buf, len)) {
 			g_warning("%s has bad TTH (len=%u): %.*s",
 				gmsg_node_infostr(n),
