@@ -52,7 +52,7 @@ void tsync_send(struct gnutella_node *n, const struct nid *node_id);
 void tsync_send_timestamp(tm_t *orig, tm_t *final);
 void tsync_got_request(struct gnutella_node *n, tm_t *got);
 void tsync_got_reply(struct gnutella_node *n,
-	tm_t *sent, tm_t *received, tm_t *replied, tm_t *got, gboolean ntp);
+	tm_t *sent, tm_t *received, tm_t *replied, tm_t *got, bool ntp);
 
 #endif /* _core_tsync_h_ */
 
