@@ -150,10 +150,12 @@ udp_get_rx_semi_reliable(enum udp_traffic utp, host_addr_t from, size_t len)
 {
 	unsigned i = 0;
 
-	if (hostiles_check(from)) {
+	if (hostiles_is_bad(from)) {
 		if (GNET_PROPERTY(udp_debug)) {
-			g_warning("UDP got %s (%zu bytes) from hostile %s -- dropped",
-				udp_traffic_to_string(utp), len, host_addr_to_string(from));
+			hostiles_flags_t flags = hostiles_check(from);
+			g_warning("UDP got %s (%zu bytes) from hostile %s (%s) -- dropped",
+				udp_traffic_to_string(utp), len, host_addr_to_string(from),
+				hostiles_flags_to_string(flags));
 		}
 		gnet_stats_inc_general(GNR_UDP_SR_RX_FROM_HOSTILE_IP);
 		return NULL;		/* Ignore message */
