@@ -45,19 +45,19 @@
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 
-#include "lib/atoms.h"
 #include "lib/ascii.h"
 #include "lib/base16.h"
 #include "lib/getdate.h"
-#include "lib/glib-missing.h"
 #include "lib/log.h"
 #include "lib/misc.h"
+#include "lib/omalloc.h"
 #include "lib/parse.h"
 #include "lib/product.h"
 #include "lib/str.h"			/* For str_bprintf()  and str_bcatf() */
-#include "lib/tm.h"
 #include "lib/timestamp.h"
+#include "lib/tm.h"
 #include "lib/utf8.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
 #define SECS_PER_DAY	86400
@@ -135,19 +135,19 @@ version_ext_str(const version_ext_t *vext, bool full)
 	bool has_extra = FALSE;
 	bool need_closing = FALSE;
 
-	rw = gm_snprintf(str, sizeof(str), "%u.%u", ver->major, ver->minor);
+	rw = str_bprintf(str, sizeof(str), "%u.%u", ver->major, ver->minor);
 
 	if (ver->patchlevel)
-		rw += gm_snprintf(&str[rw], sizeof(str)-rw, ".%u", ver->patchlevel);
+		rw += str_bprintf(&str[rw], sizeof(str)-rw, ".%u", ver->patchlevel);
 
 	if (ver->tag) {
-		rw += gm_snprintf(&str[rw], sizeof(str)-rw, "%c", ver->tag);
+		rw += str_bprintf(&str[rw], sizeof(str)-rw, "%c", ver->tag);
 		if (ver->taglevel)
-			rw += gm_snprintf(&str[rw], sizeof(str)-rw, "%u", ver->taglevel);
+			rw += str_bprintf(&str[rw], sizeof(str)-rw, "%u", ver->taglevel);
 	}
 
 	if (ver->build)
-		rw += gm_snprintf(&str[rw], sizeof(str)-rw, "-%u", ver->build);
+		rw += str_bprintf(&str[rw], sizeof(str)-rw, "-%u", ver->build);
 
 	if (vext->commit_len != 0) {
 		static char digest[SHA1_BASE16_SIZE + 1];
@@ -155,32 +155,32 @@ version_ext_str(const version_ext_t *vext, bool full)
 
 		sha1_to_base16_buf(&vext->commit, digest, sizeof digest);
 		digest[offset] = '\0';
-		rw += gm_snprintf(&str[rw], sizeof(str)-rw, "-g%s", digest);
+		rw += str_bprintf(&str[rw], sizeof(str)-rw, "-g%s", digest);
 	}
 
 	if (vext->dirty)
-		rw += gm_snprintf(&str[rw], sizeof(str)-rw, "-dirty");
+		rw += str_bprintf(&str[rw], sizeof(str)-rw, "-dirty");
 
 	if (ver->timestamp || (full && vext->osname != NULL)) {
-		rw += gm_snprintf(&str[rw], sizeof(str)-rw, " (");
+		rw += str_bprintf(&str[rw], sizeof(str)-rw, " (");
 		need_closing = TRUE;
 	}
 
 	if (ver->timestamp) {
 		struct tm *tmp = localtime(&ver->timestamp);
-		rw += gm_snprintf(&str[rw], sizeof(str)-rw, "%d-%02d-%02d",
+		rw += str_bprintf(&str[rw], sizeof(str)-rw, "%d-%02d-%02d",
 			tmp->tm_year + 1900, tmp->tm_mon + 1, tmp->tm_mday);
 		has_extra = TRUE;
 	}
 
 	if (full && vext->osname != NULL) {
 		if (has_extra)
-			rw += gm_snprintf(&str[rw], sizeof(str)-rw, "; ");
-		rw += gm_snprintf(&str[rw], sizeof(str)-rw, "%s", vext->osname);
+			rw += str_bprintf(&str[rw], sizeof(str)-rw, "; ");
+		rw += str_bprintf(&str[rw], sizeof(str)-rw, "%s", vext->osname);
 	}
 
 	if (need_closing)
-		rw += gm_snprintf(&str[rw], sizeof(str)-rw, ")");
+		rw += str_bprintf(&str[rw], sizeof(str)-rw, ")");
 
 	return str;
 }
@@ -195,23 +195,23 @@ version_str(const version_t *ver)
 	static char str[80];
 	int rw;
 
-	rw = gm_snprintf(str, sizeof(str), "%u.%u", ver->major, ver->minor);
+	rw = str_bprintf(str, sizeof(str), "%u.%u", ver->major, ver->minor);
 
 	if (ver->patchlevel)
-		rw += gm_snprintf(&str[rw], sizeof(str)-rw, ".%u", ver->patchlevel);
+		rw += str_bprintf(&str[rw], sizeof(str)-rw, ".%u", ver->patchlevel);
 
 	if (ver->tag) {
-		rw += gm_snprintf(&str[rw], sizeof(str)-rw, "%c", ver->tag);
+		rw += str_bprintf(&str[rw], sizeof(str)-rw, "%c", ver->tag);
 		if (ver->taglevel)
-			rw += gm_snprintf(&str[rw], sizeof(str)-rw, "%u", ver->taglevel);
+			rw += str_bprintf(&str[rw], sizeof(str)-rw, "%u", ver->taglevel);
 	}
 
 	if (ver->build)
-		rw += gm_snprintf(&str[rw], sizeof(str)-rw, "-%u", ver->build);
+		rw += str_bprintf(&str[rw], sizeof(str)-rw, "-%u", ver->build);
 
 	if (ver->timestamp) {
 		struct tm *tmp = localtime(&ver->timestamp);
-		rw += gm_snprintf(&str[rw], sizeof(str)-rw, " (%d-%02d-%02d)",
+		rw += str_bprintf(&str[rw], sizeof(str)-rw, " (%d-%02d-%02d)",
 			tmp->tm_year + 1900, tmp->tm_mon + 1, tmp->tm_mday);
 	}
 
@@ -587,15 +587,15 @@ version_new_found(const char *text, bool stable)
         utf8_strlcpy(last_dev, text, sizeof last_dev);
 
 	if ('\0' != last_stable[0] && '\0' != last_dev[0]) {
-		gm_snprintf(s, sizeof s,
+		str_bprintf(s, sizeof s,
 			_(" - Newer versions available: release %s / from git %s"),
 			last_stable, last_dev);
 	} else if ('\0' != last_stable[0]) {
-		gm_snprintf(s, sizeof s,
+		str_bprintf(s, sizeof s,
 			_(" - Newer version available: release %s"),
 			last_stable);
 	} else if ('\0' != last_dev[0]) {
-		gm_snprintf(s, sizeof s,
+		str_bprintf(s, sizeof s,
 			_(" - Newer version available: from git %s"),
 			last_dev);
 	}
@@ -820,7 +820,7 @@ version_build_string(void)
 		}
 #endif /* HAS_UNAME */
 
-		gm_snprintf(buf, sizeof buf,
+		str_bprintf(buf, sizeof buf,
 			"%s/%s%s (%s; %s; %s%s%s)",
 			product_get_name(), product_get_version(),
 			product_get_build_full(), product_get_date(),
@@ -840,7 +840,7 @@ version_init(void)
 {
 	time_t now;
 
-	version_string = atom_str_get(version_build_string());
+	version_string = ostrdup_readonly(version_build_string());
 	now = tm_time();
 
 	{
@@ -861,12 +861,12 @@ version_init(void)
 	{
 		char buf[128];
 
-		gm_snprintf(buf, sizeof(buf),
+		str_bprintf(buf, sizeof(buf),
 			"%s/%s%s (%s)",
 			product_get_name(), product_get_version(),
 			product_get_build_full(), product_get_date());
 
-		version_short_string = atom_str_get(buf);
+		version_short_string = ostrdup_readonly(buf);
 	}
 
 	last_rel_version = our_version;			/* struct copy */
@@ -1040,11 +1040,6 @@ version_string_dump(void)
 G_GNUC_COLD void
 version_close(void)
 {
-	atom_str_free(version_string);
-	version_string = NULL;
-	atom_str_free(version_short_string);
-	version_short_string = NULL;
-
 	if (version_cmp(&our_version, &last_rel_version) < 0)
 		g_warning("upgrade recommended: most recent released version seen: %s",
 			version_str(&last_rel_version));
