@@ -935,9 +935,11 @@ handle_proxy_ack(struct gnutella_node *n,
 			host_addr_port_to_string(ha, port), vmsg->name, node_infostr(n));
 		return;
 	}
-	if (hostiles_check(ha)) {
-		g_message("VMSG got proxy ACK from hostile %s: proxy at %s",
-			node_infostr(n), host_addr_port_to_string(ha, port));
+	if (hostiles_is_bad(ha)) {
+		hostiles_flags_t flags = hostiles_check(ha);
+		g_message("VMSG ignoring proxy ACK from hostile %s (%s): proxy at %s",
+			node_infostr(n), hostiles_flags_to_string(flags),
+			host_addr_port_to_string(ha, port));
 		return;
 	}
 
