@@ -55,10 +55,10 @@
 
 void kmsg_send_ping(knode_t *kn, const guid_t *muid);
 void kmsg_send_find_node(knode_t *kn, const kuid_t *id, const guid_t *muid,
-	pmsg_free_t mfree, gpointer marg);
+	pmsg_free_t mfree, void *marg);
 void kmsg_send_find_value(knode_t *kn, const kuid_t *id, dht_value_type_t type,
 	kuid_t **skeys, int scnt,
-	const guid_t *muid, pmsg_free_t mfree, gpointer marg);
+	const guid_t *muid, pmsg_free_t mfree, void *marg);
 GSList *kmsg_build_store(
 	const void *token, size_t toklen, dht_value_t **vvec, int vcnt);
 	
