@@ -34,26 +34,33 @@
 #include "common.h"
 
 #include "rpc.h"
-#include "routing.h"
 #include "kmsg.h"
 #include "knode.h"
+#include "routing.h"
+#include "stable.h"
 
 #include "if/gnet_property_priv.h"
+#include "if/dht/kuid.h"
 
 #include "core/guid.h"
 #include "core/gnet_stats.h"
 
+#include "lib/aging.h"
 #include "lib/atoms.h"
 #include "lib/cq.h"
+#include "lib/gnet_host.h"
 #include "lib/hikset.h"
 #include "lib/host_addr.h"
+#include "lib/stacktrace.h"		/* For stacktrace_function_name() */
 #include "lib/tm.h"
 #include "lib/walloc.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
-enum rpc_cb_magic { RPC_CB_MAGIC = 0x74c8b10U };
+#define DHT_RPC_RECENT_KEEP	(5*60)	/* 5 minutes */
+#define DHT_RPC_LINGER_MS	15000 	/* ms, 15 seconds */
 
+enum rpc_cb_magic { RPC_CB_MAGIC = 0x74c8b10U };
 
 /**
  * An RPC callback descriptor.
@@ -62,6 +69,7 @@ struct rpc_cb {
 	enum rpc_cb_magic magic;	/**< magic */
 	enum dht_rpc_op op;			/**< Operation type */
 	host_addr_t addr;			/**< The host from which we expect a reply */
+	uint16 port;				/**< The port from which we expect a reply */
 	tm_t start;					/**< The time at which we initiated the RPC */
 	const guid_t *muid;			/**< MUID of the message sent (atom) */
 	knode_t *kn;				/**< Remote node to which RPC was sent */
@@ -69,6 +77,7 @@ struct rpc_cb {
 	dht_rpc_cb_t cb;			/**< Callback routine to invoke */
 	void *arg;					/**< Additional opaque argument */
 	cevent_t *timeout;			/**< Callout queue timeout event */
+	unsigned lingering:1;		/**< RPC was cancelled / timed out */
 };
 
 static inline void
@@ -81,6 +90,12 @@ rpc_cb_check(const struct rpc_cb * const rcb)
 
 static hikset_t *pending;		/**< Pending RPC (GUID -> rpc_cb) */
 
+/**
+ * Table recording the mappings between a KUID and an IP:port, as validated
+ * through an RPC exchange.
+ */
+static aging_table_t *rpc_recent;
+
 /**
  * RPC operation to string, for logs.
  */
@@ -97,6 +112,146 @@ op_to_string(enum dht_rpc_op op)
 	return "UNKNOWN";
 }
 
+/**
+ * Free an entry in the `rpc_recent' aging table.
+ */
+static void
+rpc_free_kuid_addr(void *key, void *value)
+{
+	kuid_atom_free(key);
+	gnet_host_free(value);
+}
+
+/**
+ * Insert entry in the `rpc_recent' aging table, recording the IP:port used
+ * by given KUID.
+ */
+static void
+dht_record_contact(const kuid_t *kuid, host_addr_t addr, uint16 port)
+{
+	gnet_host_t *host;
+
+	host = aging_lookup_revitalise(rpc_recent, kuid);
+
+	if (host != NULL) {
+		if (
+			port == gnet_host_get_port(host) &&
+			host_addr_equal(addr, gnet_host_get_addr(host))
+		)
+			return;
+		aging_remove(rpc_recent, kuid);
+	}
+
+	aging_insert(rpc_recent, kuid_get_atom(kuid), gnet_host_new(addr, port));
+
+	/*
+	 * Update the count, knowing that it can decrease without us knowing
+	 * immediately since there is no hook that can be plugged on the internal
+	 * cleanup that the aging table routinely performs.
+	 */
+
+	gnet_stats_set_general(GNR_DHT_RPC_RECENT_NODES_HELD,
+		aging_count(rpc_recent));
+}
+
+/**
+ * Fix given IP:port if we know that the KUID maps to a different address
+ * using the recent RPC information.
+ *
+ * @param kuid		the KUID of the node
+ * @param addr		pointer to the address we want to check / update
+ * @param port		pointer to the port we want to check / update
+ * @param source	origin of the KUID, for logging purposes
+ *
+ * @return TRUE if IP:port was updated.
+ */
+bool
+dht_fix_kuid_contact(const kuid_t *kuid, host_addr_t *addr, uint16 *port,
+	const char *source)
+{
+	gnet_host_t *host;
+
+	/*
+	 * If we had a recent RPC transaction with this KUID, we may know it under
+	 * a different IP:port.
+	 */
+
+	host = aging_lookup(rpc_recent, kuid);
+
+	if (host != NULL) {
+		host_addr_t xaddr = gnet_host_get_addr(host);
+		uint16 xport = gnet_host_get_port(host);
+
+		if (xport == *port && host_addr_equal(xaddr, *addr))
+			return FALSE;
+
+		if (GNET_PROPERTY(dht_lookup_debug)) {
+			g_warning("DHT fixing contact address (%s) for kuid=%s"
+				" from %s to %s (using recent RPC info)",
+				source, kuid_to_hex_string(kuid),
+				host_addr_port_to_string(*addr, *port),
+				host_addr_port_to_string2(xaddr, xport));
+		}
+
+		*addr = xaddr;		/* Struct copy */
+		*port = xport;
+		return TRUE;
+	}
+	
+	return FALSE;
+}
+
+/**
+ * Fix a knode in-place by changing its IP:port if we know that its KUID
+ * maps to a different value from the RPC recent info or from the routing
+ * table.
+ *
+ * @param kn		the node contact
+ * @param source	origin of the node, for logging purposes
+ *
+ * @return TRUE if contact was updated.
+ */
+bool
+dht_fix_contact(knode_t *kn, const char *source)
+{
+	knode_t *rn;
+
+	g_assert(!knode_is_shared(kn, FALSE));	/* Since addr:port can be patched */
+
+	/*
+	 * First look using the recent RPC information.
+	 */
+
+	if (dht_fix_kuid_contact(kn->id, &kn->addr, &kn->port, source))
+		return TRUE;
+
+	/*
+	 * If we hold the node in the routing table and already did a successful
+	 * RPC exchange with it, compare the addresses.
+	 */
+
+	rn = dht_find_node(kn->id);
+
+	if (rn != NULL && (rn->flags & KNODE_F_RPC)) {
+		if (rn->port == kn->port && host_addr_equal(rn->addr, kn->addr))
+			return FALSE;
+
+		if (GNET_PROPERTY(dht_lookup_debug)) {
+			g_warning("DHT fixing contact address (%s) for kuid=%s"
+				" from %s to %s (using routing table)",
+				source, kuid_to_hex_string(kn->id),
+				host_addr_port_to_string(kn->addr, kn->port),
+				host_addr_port_to_string(rn->addr, rn->port));
+		}
+
+		kn->addr = rn->addr;	/* Struct copy */
+		kn->port = rn->port;
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 /**
  * Initialize the RPC layer.
  */
@@ -107,6 +262,9 @@ dht_rpc_init(void)
 
 	pending = hikset_create(
 		offsetof(struct rpc_cb, muid), HASH_KEY_FIXED, GUID_RAW_SIZE);
+
+	rpc_recent = aging_make(DHT_RPC_RECENT_KEEP,
+		kuid_hash, kuid_eq, rpc_free_kuid_addr);
 }
 
 /**
@@ -118,9 +276,14 @@ rpc_cb_free(struct rpc_cb *rcb, bool in_shutdown)
 	rpc_cb_check(rcb);
 
 	if (in_shutdown) {
-		knode_rpc_dec(rcb->kn);
-		if (rcb->cb)
-			(*rcb->cb)(DHT_RPC_TIMEOUT, rcb->kn, NULL, 0, NULL, 0, rcb->arg);
+		/* Lingering RPCs have already invoked their callback */
+		if (!rcb->lingering) {
+			knode_rpc_dec(rcb->kn);
+			if (rcb->cb != NULL) {
+				(*rcb->cb)(DHT_RPC_TIMEOUT,
+					rcb->kn, NULL, 0, NULL, 0, rcb->arg);
+			}
+		}
 	} else {
 		hikset_remove(pending, rcb->muid);
 	}
@@ -168,16 +331,38 @@ rpc_delay(const knode_t *kn)
 }
 
 /**
- * Generic RPC operation timeout (callout queue callback).
+ * End of RPC lingering time (callout queue callback).
  */
 static void
-rpc_timed_out(cqueue_t *unused_cq, void *obj)
+rpc_lingered(cqueue_t *unused_cq, void *obj)
 {
 	struct rpc_cb *rcb = obj;
 
 	rpc_cb_check(rcb);
 	(void) unused_cq;
 
+	if (GNET_PROPERTY(dht_rpc_debug) > 5) {
+		g_debug("DHT RPC %s #%s finished lingering",
+			op_to_string(rcb->op), guid_to_string(rcb->muid));
+	}
+
+	rcb->timeout = NULL;
+	rpc_cb_free(rcb, FALSE);
+}
+
+/**
+ * Generic RPC operation timeout (callout queue callback).
+ */
+static void
+rpc_timed_out(cqueue_t *cq, void *obj)
+{
+	struct rpc_cb *rcb = obj;
+
+	rpc_cb_check(rcb);
+
+	if (cq != NULL)
+		gnet_stats_inc_general(GNR_DHT_RPC_TIMED_OUT);
+
 	rcb->timeout = NULL;
 	dht_node_timed_out(rcb->kn);
 
@@ -188,10 +373,26 @@ rpc_timed_out(cqueue_t *unused_cq, void *obj)
 
 	knode_rpc_dec(rcb->kn);
 
-	if (rcb->cb)
+	if (rcb->cb != NULL) {
+		if (GNET_PROPERTY(dht_rpc_debug) > 4) {
+			g_debug("DHT RPC %s #%s invoking %s(TIMEOUT, %p)",
+				op_to_string(rcb->op), guid_to_string(rcb->muid),
+				stacktrace_function_name(rcb->cb), rcb->arg);
+		}
 		(*rcb->cb)(DHT_RPC_TIMEOUT, rcb->kn, NULL, 0, NULL, 0, rcb->arg);
+	} else {
+		if (GNET_PROPERTY(dht_rpc_debug) > 4) {
+			g_debug("DHT RPC %s #%s timed out",
+				op_to_string(rcb->op), guid_to_string(rcb->muid));
+		}
+	}
 
-	rpc_cb_free(rcb, FALSE);
+	/*
+	 * Linger for a while to see how many "late" replies we get.
+	 */
+
+	rcb->timeout = cq_main_insert(DHT_RPC_LINGER_MS, rpc_lingered, rcb);
+	rcb->lingering = TRUE;
 }
 
 /**
@@ -239,13 +440,14 @@ rpc_call_prepare(
 	 * Create and fill the RPC control block.
 	 */
 
-	WALLOC(rcb);
+	WALLOC0(rcb);
 	rcb->magic = RPC_CB_MAGIC;
 	rcb->op = op;
 	rcb->kn = knode_refcnt_inc(kn);
 	rcb->flags = flags;
 	rcb->muid = atom_guid_get(&muid);
 	rcb->addr = kn->addr;
+	rcb->port = kn->port;
 	rcb->timeout = cq_main_insert(delay, rpc_timed_out, rcb);
 	rcb->cb = cb;
 	rcb->arg = arg;
@@ -253,10 +455,52 @@ rpc_call_prepare(
 	knode_rpc_inc(kn);
 
 	hikset_insert_key(pending, &rcb->muid);
+	gnet_stats_inc_general(GNR_DHT_RPC_MSG_PREPARED);
+
+	if (GNET_PROPERTY(dht_rpc_debug) > 4) {
+		g_debug("DHT RPC created %s #%s to %s with callback %s(%p), "
+			"timeout %d ms",
+			op_to_string(rcb->op), guid_to_string(rcb->muid),
+			knode_to_string(kn), stacktrace_function_name(cb), arg, delay);
+	}
 
 	return rcb->muid;
 }
 
+/**
+ * Force a timeout on the RPC.
+ *
+ * @return whether we found the MUID to time out.
+ */
+bool
+dht_rpc_timeout(const guid_t *muid)
+{
+	struct rpc_cb *rcb;
+
+	rcb = hikset_lookup(pending, muid);
+	if (NULL == rcb)
+		return FALSE;
+
+	rpc_cb_check(rcb);
+
+	if (rcb->lingering) {
+		cq_expire(rcb->timeout);
+		return FALSE;		/* Already timed out since we're lingering */
+	}
+
+	if (GNET_PROPERTY(dht_rpc_debug)) {
+		g_debug("DHT RPC forcing timeout of %s #%s to %s",
+			op_to_string(rcb->op), guid_to_string(rcb->muid),
+			knode_to_string(rcb->kn));
+	}
+
+	gnet_stats_inc_general(GNR_DHT_RPC_MSG_CANCELLED);
+	cq_cancel(&rcb->timeout);
+	rpc_timed_out(NULL, rcb);
+
+	return TRUE;
+}
+
 /**
  * Cancel an RPC.
  *
@@ -271,10 +515,24 @@ dht_rpc_cancel(const guid_t *muid)
 	struct rpc_cb *rcb;
 
 	rcb = hikset_lookup(pending, muid);
-	if (!rcb)
+	if (NULL == rcb)
 		return FALSE;
 
 	rpc_cb_check(rcb);
+
+	if (rcb->lingering) {
+		cq_expire(rcb->timeout);
+		return FALSE;		/* Already timed out since we're lingering */
+	}
+
+	if (GNET_PROPERTY(dht_rpc_debug) > 3) {
+		g_debug("DHT RPC cancelling %s #%s to %s -- not calling %s(%p)",
+			op_to_string(rcb->op), guid_to_string(rcb->muid),
+			knode_to_string(rcb->kn),
+			stacktrace_function_name(rcb->cb), rcb->arg);
+	}
+
+	gnet_stats_inc_general(GNR_DHT_RPC_MSG_CANCELLED);
 	knode_rpc_dec(rcb->kn);
 	rpc_cb_free(rcb, FALSE);
 	return TRUE;
@@ -292,11 +550,23 @@ dht_rpc_cancel_if_no_callback(const guid_t *muid)
 	struct rpc_cb *rcb;
 
 	rcb = hikset_lookup(pending, muid);
-	if (!rcb)
+	if (NULL == rcb)
 		return FALSE;
 
 	rpc_cb_check(rcb);
+
+	if (rcb->lingering) {
+		cq_expire(rcb->timeout);
+		return FALSE;		/* Already timed out since we're lingering */
+	}
+
 	if (NULL == rcb->cb) {
+		if (GNET_PROPERTY(dht_rpc_debug) > 3) {
+			g_debug("DHT RPC cancelling %s #%s to %s with no callback",
+				op_to_string(rcb->op), guid_to_string(rcb->muid),
+				knode_to_string(rcb->kn));
+		}
+
 		knode_rpc_dec(rcb->kn);
 		rpc_cb_free(rcb, FALSE);
 		return TRUE;
@@ -318,16 +588,33 @@ dht_rpc_info(const guid_t *muid, host_addr_t *addr, uint16 *port)
 	knode_t *rn;
 
 	rcb = hikset_lookup(pending, muid);
-	if (!rcb)
+	if (NULL == rcb)
 		return FALSE;
 
 	rpc_cb_check(rcb);
 
+	/*
+	 * Note that we're not checking for lingering RPCs here because we
+	 * are not handling the RPC itself, we're trying to fixup the contact
+	 * address.  If we find a lingering RPC, it means we got a late reply,
+	 * but we can nonethless update the contact.
+	 */
+
 	rn = rcb->kn;
 	knode_check(rn);
 
-	if (addr) *addr = rn->addr;
-	if (port) *port = rn->port;
+	if (
+		GNET_PROPERTY(dht_rpc_debug) &&
+		(rn->port != rcb->port || !host_addr_equal(rn->addr, rcb->addr))
+	) {
+		g_warning("DHT RPC had sent %s #%s to %s, now is %s",
+			op_to_string(rcb->op), guid_to_string(rcb->muid),
+			host_addr_port_to_string(rcb->addr, rcb->port),
+			knode_to_string(rn));
+	}
+
+	if (addr) *addr = rcb->addr;
+	if (port) *port = rcb->port;
 
 	return TRUE;
 }
@@ -359,11 +646,50 @@ dht_rpc_answer(const guid_t *muid,
 	knode_check(kn);
 
 	rcb = hikset_lookup(pending, muid);
-	if (!rcb)
+	if (NULL == rcb)
 		return FALSE;
 
 	rpc_cb_check(rcb);
-	cq_cancel(&rcb->timeout);
+
+	if (GNET_PROPERTY(dht_rpc_debug) > 2) {
+		g_debug("DHT RPC got %sanswer to %s #%s sent to %s, timeout in %s ms",
+			rcb->lingering ? "late " : "",
+			op_to_string(rcb->op), guid_to_string(rcb->muid),
+			knode_to_string(rcb->kn),
+			cq_time_to_string(cq_remaining(rcb->timeout)));
+	}
+
+	/*
+	 * If this is a late reply (received whilst lingering), do not handle
+	 * the RPC.  We can stop the lingering process though since we got
+	 * a reply and we don't expect more.
+	 */
+
+	if (rcb->lingering) {
+		gnet_stats_inc_general(GNR_DHT_RPC_LATE_REPLIES_RECEIVED);
+
+		if (GNET_PROPERTY(dht_rpc_debug) > 1) {
+			g_debug("DHT RPC late reply for %s #%s to %s",
+				op_to_string(rcb->op), guid_to_string(rcb->muid),
+				knode_to_string(rcb->kn));
+		}
+
+		/*
+		 * If the node from which we got a reply is in the routing table,
+		 * update the RTT EMA, since it took longer than expected to get a
+		 * reply -- we want to do better next time at projecting a suitable RTT.
+		 */
+
+		if (KNODE_UNKNOWN != kn->status) {
+			tm_now_exact(&now);
+			kn->rtt += (tm_elapsed_ms(&now, &rcb->start) >> 1) - (kn->rtt >> 1);
+		}
+
+		cq_expire(rcb->timeout);		/* Will free up `rcb' */
+		return FALSE;
+	}
+
+	cq_cancel(&rcb->timeout);	/* This is an RPC timeout, not a lingering */
 
 	/*
 	 * The node that was registered during the creation of the RPC was
@@ -390,24 +716,31 @@ dht_rpc_answer(const guid_t *muid,
 		/*
 		 * This node is stale: the node to which we sent the RPC bears
 		 * a KUID different from the one we thought it had.  The node we
-		 * knew about is therefore gone.
+		 * knew about is therefore gone and has a new KUID.
 		 *
 		 * Remove the original node from the routing table (in case it is
-		 * present) and do not handle the reply.
+		 * present) and do not handle the reply: that would be misleading
+		 * if we are performing a node lookup for instance, because we do not
+		 * want to enter the new node in the path.
 		 */
 
-		gnet_stats_count_general(GNR_DHT_RPC_KUID_REPLY_MISMATCH, 1);
+		gnet_stats_inc_general(GNR_DHT_RPC_KUID_REPLY_MISMATCH);
 
-		if (GNET_PROPERTY(dht_debug)) {
-			g_debug("DHT sent %s RPC %s to %s but got reply from %s",
+		if (GNET_PROPERTY(dht_debug) || GNET_PROPERTY(dht_rpc_debug)) {
+			g_debug("DHT RPC sent %s #%s to %s but got reply from %s via %s",
 				op_to_string(rcb->op),
 				guid_to_string(rcb->muid),
 				knode_to_string(rn),
-				knode_to_string2(kn));
+				knode_to_string2(kn), node_infostr(n));
 		}
 
-		dht_remove_node(rn);				/* Discard obsolete entry */
-		rpc_timed_out(cq_main(), rcb);		/* Invoke user callback if any */
+		/* Mark alive for stable_replace() */
+		rn->flags |= KNODE_F_ALIVE;		/* Receiving RPC reply traffic */
+		kn->flags |= KNODE_F_ALIVE;		/* Got traffic from that node */
+
+		stable_replace(rn, kn);			/* KUID of rn was changed */
+		dht_remove_node(rn);			/* Remove obsolete entry from routing */
+		rpc_timed_out(cq_main(), rcb);	/* Invoke user callback if any */
 
 		return FALSE;	/* RPC was sent to wrong node, ignore */
 	}
@@ -461,11 +794,22 @@ dht_rpc_answer(const guid_t *muid,
 	 * found at RPC reply time, a new node in "unknown" status was created.
 	 */
 
-	if (kn->status == KNODE_STALE) {
+	if (KNODE_STALE == kn->status) {
 		dht_set_node_status(kn, KNODE_GOOD);
-		gnet_stats_count_general(GNR_DHT_REVITALIZED_STALE_NODES, 1);
+		gnet_stats_inc_general(GNR_DHT_REVITALIZED_STALE_NODES);
 	}
 
+	/*
+	 * Record the IP:port associated with this KUID so that we may fixup the
+	 * KUID => address mappings when performing FIND_NODE operations.
+	 *
+	 * We use the IP:port to which we sent the RPC, since we got a good reply
+	 * by sending something there.
+	 */
+
+	if (!(rcb->flags & RPC_CALL_NO_VERIFY) || kuid_eq(kn->id, rn->id))
+		dht_record_contact(kn->id, rcb->addr, rcb->port);
+
 	/*
 	 * Invoke user callback, if any configured.
 	 * The amount of pending RPCs is decreased before invoking the callback.
@@ -473,10 +817,17 @@ dht_rpc_answer(const guid_t *muid,
 
 	knode_rpc_dec(rcb->kn);
 
-	if (rcb->cb)
+	if (rcb->cb != NULL) {
+		if (GNET_PROPERTY(dht_rpc_debug) > 4) {
+			g_debug("DHT RPC %s #%s invoking %s(REPLY, %s, %zu byte%s, %p)",
+				op_to_string(rcb->op), guid_to_string(rcb->muid),
+				stacktrace_function_name(rcb->cb), kmsg_name(function),
+				len, 1 == len ? "" : "s", rcb->arg);
+		}
 		(*rcb->cb)(DHT_RPC_REPLY, rn, n, function, payload, len, rcb->arg);
+	}
 
-	rpc_cb_free(rcb, FALSE);
+	rpc_cb_free(rcb, FALSE);		/* Got a reply, no need to linger */
 	return TRUE;
 }
 
@@ -532,7 +883,7 @@ dht_lazy_rpc_ping(knode_t *kn)
 				knode_to_string(kn), kn->rpc_pending,
 				1 == kn->rpc_pending ? "" : "s");
 		}
-		gnet_stats_count_general(GNR_DHT_ALIVE_PINGS_AVOIDED, 1);
+		gnet_stats_inc_general(GNR_DHT_ALIVE_PINGS_AVOIDED);
 		return FALSE;
 	} else {
 		dht_rpc_ping(kn, NULL, NULL);
@@ -658,6 +1009,7 @@ dht_rpc_close(void)
 {
 	hikset_foreach(pending, rpc_free_kv, NULL);
 	hikset_free_null(&pending);
+	aging_destroy(&rpc_recent);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
