@@ -50,7 +50,6 @@
 #include "if/core/net_stats.h"
 #include "if/core/hcache.h"
 #include "if/core/hsep.h"
-#include "if/core/http.h"
 #include "if/core/parq.h"
 #include "if/core/search.h"
 #include "if/core/share.h"
@@ -512,20 +511,6 @@ guc_hsep_remove_global_table_listener(GCallback cb)
 }
 
 
-/*	HTTP interface functions (UI -> Core)*/
-const char *
-guc_http_range_to_string(const GSList *list)
-{
-	return http_range_to_string(list);
-}
-
-GSList *
-guc_http_range_merge(GSList *old_list, GSList *new_list)
-{
-	return http_range_merge(old_list, new_list);
-}
-
-
 /*	node interface functions (UI -> Core)*/
 void
 guc_node_add_node_added_listener(node_added_listener_t l)
