@@ -1,6 +1,4 @@
 /*
- * $Id: parq.h 6513 2004-08-27 15:27:06Z rmanfredi $
- *
  * Copyright (c) 2003, Jeroen Asselman
  *
  *----------------------------------------------------------------------
@@ -34,10 +32,10 @@
 
 struct download;
 
-gint get_parq_dl_position(const struct download *d);
-gint get_parq_dl_queue_length(const struct download *d);
-gint get_parq_dl_eta(const struct download *d);
-gint get_parq_dl_retry_delay(const struct download *d);
+int get_parq_dl_position(const struct download *d);
+int get_parq_dl_queue_length(const struct download *d);
+int get_parq_dl_eta(const struct download *d);
+int get_parq_dl_retry_delay(const struct download *d);
 
 #endif /* CORE_SOURCES */
 #endif /* _if_core_parq_h_ */
