@@ -43,7 +43,7 @@ void arc4random_addrandom(const unsigned char *dat, int datlen);
 #endif
 
 void arc4random_stir_once(void);
-uint64 arc4random_upto64(uint64 max);
+uint64 arc4random64(void);
 
 #endif /* _arc4random_h_ */
 
