@@ -41,10 +41,10 @@
 
 #include "override.h"			/* Must be the last header included */
 
-gboolean
+bool
 compat_is_superuser(void)
 {
-	gboolean ret = FALSE;	/* Assume luser by default */
+	bool ret = FALSE;	/* Assume luser by default */
 	
 #ifdef HAS_GETUID
 	ret |= 0 == getuid();
@@ -57,7 +57,7 @@ compat_is_superuser(void)
 	return ret;
 }
 
-gboolean
+bool
 compat_process_is_alive(pid_t pid)
 {
 #ifdef MINGW32
@@ -76,7 +76,10 @@ compat_process_is_alive(pid_t pid)
 int
 compat_daemonize(const char *directory)
 {
-#ifndef MINGW32	/* FIXME MINGW32 */
+#ifdef MINGW32
+	/* FIXME MINGW32 */
+	(void) directory;
+#else
 	pid_t pid;
 	int i;
 
@@ -140,7 +143,7 @@ compat_daemonize(const char *directory)
 		g_warning("freopen() failed for stderr");
 		return -1;
 	}
-#endif	/* !MINGW32 */
+#endif	/* MINGW32 */
 	return 0;
 }
 
