@@ -668,13 +668,12 @@ elist_merge_sort(elist_t *list, link_t *sublist, size_t count,
 		int c = (*cmp)(d1, d2, data);
 
 		if (c <= 0) {
-			l->next = l1;
+			l = l->next = l1;
 			l1 = l1->next;
 		} else {
-			l->next = l2;
+			l = l->next = l2;
 			l2 = l2->next;
 		}
-		l = l->next;
 		l->prev = prev;
 		prev = l;
 	}
