@@ -44,106 +44,106 @@
  * or little-endian.
  */
 
-static inline ALWAYS_INLINE G_GNUC_PURE guint8
+static inline ALWAYS_INLINE G_GNUC_PURE uint8
 peek_u8(const void *p)
 {
-	const guint8 *q = p;
+	const uint8 *q = p;
 	return q[0];
 }
 
-static inline ALWAYS_INLINE G_GNUC_PURE guint16
+static inline ALWAYS_INLINE G_GNUC_PURE uint16
 peek_u16(const void *p)
 {
 	const unsigned char *q = p;
-	guint16 v;
+	uint16 v;
 	memcpy(&v, q, sizeof v);
 	return v;
 }
 
-static inline ALWAYS_INLINE G_GNUC_PURE guint32
+static inline ALWAYS_INLINE G_GNUC_PURE uint32
 peek_u32(const void *p)
 {
 	const unsigned char *q = p;
-	guint32 v;
+	uint32 v;
 	memcpy(&v, q, sizeof v);
 	return v;
 }
 
-static inline ALWAYS_INLINE G_GNUC_PURE guint64
+static inline ALWAYS_INLINE G_GNUC_PURE uint64
 peek_u64(const void *p)
 {
 	const unsigned char *q = p;
-	guint64 v;
+	uint64 v;
 	memcpy(&v, q, sizeof v);
 	return v;
 }
 
-static inline G_GNUC_PURE guint16
+static inline G_GNUC_PURE uint16
 peek_be16(const void *p)
 {
 	const unsigned char *q = p;
-	guint16 v;
+	uint16 v;
 
-#if G_BYTE_ORDER == G_BIG_ENDIAN
+#if IS_BIG_ENDIAN
 	memcpy(&v, q, sizeof v);
 #else
-	v = ((guint16) peek_u8(q) << 8) | peek_u8(&q[sizeof v / 2]);
+	v = ((uint16) peek_u8(q) << 8) | peek_u8(&q[sizeof v / 2]);
 #endif
 	return v;
 }
 
-static inline G_GNUC_PURE guint32
+static inline G_GNUC_PURE uint32
 peek_be32(const void *p)
 {
 	const unsigned char *q = p;
-	guint32 v;
+	uint32 v;
 
-#if G_BYTE_ORDER == G_BIG_ENDIAN
+#if IS_BIG_ENDIAN
 	memcpy(&v, q, sizeof v);
 #else
-	v = ((guint32) peek_be16(q) << 16) | peek_be16(&q[sizeof v / 2]);
+	v = ((uint32) peek_be16(q) << 16) | peek_be16(&q[sizeof v / 2]);
 #endif
 	return v;
 }
 
-static inline G_GNUC_PURE guint64
+static inline G_GNUC_PURE uint64
 peek_be64(const void *p)
 {
 	const unsigned char *q = p;
-	guint64 v;
+	uint64 v;
 
-#if G_BYTE_ORDER == G_BIG_ENDIAN
+#if IS_BIG_ENDIAN
 	memcpy(&v, q, sizeof v);
 #else
-	v = ((guint64) peek_be32(q) << 32) | peek_be32(&q[sizeof v / 2]);
+	v = ((uint64) peek_be32(q) << 32) | peek_be32(&q[sizeof v / 2]);
 #endif
 	return v;
 }
 
-static inline G_GNUC_PURE guint16
+static inline G_GNUC_PURE uint16
 peek_le16(const void *p)
 {
 	const unsigned char *q = p;
-	guint16 v;
+	uint16 v;
 
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+#if IS_LITTLE_ENDIAN
 	memcpy(&v, q, sizeof v);
 #else
-	v = peek_u8(q) | ((guint16) peek_u8(&q[sizeof v / 2]) << 8);
+	v = peek_u8(q) | ((uint16) peek_u8(&q[sizeof v / 2]) << 8);
 #endif
 	return v;
 }
 
-static inline G_GNUC_PURE guint32
+static inline G_GNUC_PURE uint32
 peek_le32(const void *p)
 {
 	const unsigned char *q = p;
-	guint32 v;
+	uint32 v;
 
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+#if IS_LITTLE_ENDIAN
 	memcpy(&v, q, sizeof v);
 #else
-	v = peek_le16(q) | ((guint32) peek_le16(&q[sizeof v / 2]) << 16);
+	v = peek_le16(q) | ((uint32) peek_le16(&q[sizeof v / 2]) << 16);
 #endif
 	return v;
 }
@@ -153,20 +153,20 @@ peek_le32(const void *p)
  * written bytes.
  */
 
-static inline ALWAYS_INLINE gpointer
-poke_u8(gpointer p, unsigned char v)
+static inline ALWAYS_INLINE void *
+poke_u8(void *p, unsigned char v)
 {
 	unsigned char *q = p;
 	*q = v & 0xff;
 	return &q[sizeof v];
 }
 
-static inline gpointer
-poke_be16(gpointer p, guint16 v)
+static inline void *
+poke_be16(void *p, uint16 v)
 {
 	unsigned char *q = p;
 
-#if G_BYTE_ORDER == G_BIG_ENDIAN
+#if IS_BIG_ENDIAN
 	memcpy(q, &v, sizeof v);
 #else
 	poke_u8(&q[0], v >> 8);
@@ -176,12 +176,12 @@ poke_be16(gpointer p, guint16 v)
 	return &q[sizeof v];
 }
 
-static inline gpointer
-poke_be32(gpointer p, guint32 v)
+static inline void *
+poke_be32(void *p, uint32 v)
 {
 	unsigned char *q = p;
 
-#if G_BYTE_ORDER == G_BIG_ENDIAN
+#if IS_BIG_ENDIAN
 	memcpy(q, &v, sizeof v);
 #else
 	poke_be16(&q[0], v >> 16);
@@ -191,12 +191,12 @@ poke_be32(gpointer p, guint32 v)
 	return &q[sizeof v];
 }
 
-static inline gpointer
-poke_be64(gpointer p, guint64 v)
+static inline void *
+poke_be64(void *p, uint64 v)
 {
 	unsigned char *q = p;
 
-#if G_BYTE_ORDER == G_BIG_ENDIAN
+#if IS_BIG_ENDIAN
 	memcpy(q, &v, sizeof v);
 #else
 	poke_be32(&q[0], v >> 32);
@@ -206,12 +206,12 @@ poke_be64(gpointer p, guint64 v)
 	return &q[sizeof v];
 }
 
-static inline gpointer
-poke_le16(gpointer p, guint16 v)
+static inline void *
+poke_le16(void *p, uint16 v)
 {
 	unsigned char *q = p;
 
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+#if IS_LITTLE_ENDIAN
 	memcpy(q, &v, sizeof v);
 #else
 	poke_u8(&q[0], v);
@@ -221,12 +221,12 @@ poke_le16(gpointer p, guint16 v)
 	return &q[sizeof v];
 }
 
-static inline gpointer
-poke_le32(gpointer p, guint32 v)
+static inline void *
+poke_le32(void *p, uint32 v)
 {
 	unsigned char *q = p;
 
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+#if IS_LITTLE_ENDIAN
 	memcpy(q, &v, sizeof v);
 #else
 	poke_le16(&q[0], v);
@@ -236,12 +236,12 @@ poke_le32(gpointer p, guint32 v)
 	return &q[sizeof v];
 }
 
-static inline gpointer
-poke_le64(gpointer p, guint64 v)
+static inline void *
+poke_le64(void *p, uint64 v)
 {
 	unsigned char *q = p;
 
-#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+#if IS_LITTLE_ENDIAN
 	memcpy(q, &v, sizeof v);
 #else
 	poke_le32(&q[0], v);
@@ -251,63 +251,44 @@ poke_le64(gpointer p, guint64 v)
 	return &q[sizeof v];
 }
 
-
+#ifdef USE_IEEE754_FLOAT
 /*
- * The list of architectures is taken from xdr_float.c of
- * SUN's XDR implementation.
- */
-#if \
-	defined(__alpha__) || \
-	defined(__arm__) || \
-	defined(__hppa__) || \
-	defined(__i386__) || \
-	defined(__ia64__) || \
-	defined(__m68k__) || \
-	defined(__mips__) || \
-	defined(__ns32k__) || \
-	defined(__powerpc__) || \
-	defined(__ppc__) || \
-	defined(__s390__) || \
-	defined(__sh__) || \
-	defined(__sparc__) || \
-	defined(__sparc) || \
-	defined(__x86_64__)
-#define FLOAT_USES_IEEE754
-#else
-#undef FLOAT_USES_IEEE754
-#error "This architecture may be unsupported. float must use IEEE 754."
-#endif
-
-#ifdef FLOAT_USES_IEEE754
-/*
- * The following routines assume integer byte order is float byte order.
  * Handles IEC 60559/IEEE 754 floating point single-precision (32-bit).
  */
 
-static inline gpointer
-poke_float_be32(gpointer p, float v)
+static inline void *
+poke_float_be32(void *p, float v)
 {
-	guint32 tmp;
+	uint32 tmp;
 
 	STATIC_ASSERT(sizeof(float) == 4);
 
 	memcpy(&tmp, &v, 4);
+#if BYTEORDER == IEEE754_BYTEORDER
 	return poke_be32(p, tmp);
+#else
+	return poke_le32(p, tmp);	/* Will swap the ordering */
+#endif
 }
 
 static inline G_GNUC_PURE float
 peek_float_be32(const void *p)
 {
-	guint32 tmp;
+	uint32 tmp;
 	float v;
 
 	STATIC_ASSERT(sizeof(float) == 4);
 
+#if BYTEORDER == IEEE754_BYTEORDER
 	tmp = peek_be32(p);
+#else
+	tmp = peek_le32(p);			/* Will swap the ordering */
+#endif
 	memcpy(&v, &tmp, 4);
 	return v;
 }
-#endif	/* FLOAT_USES_IEEE754 */
+#endif	/* USE_IEEE754_FLOAT */
 
 #endif /* _endian_h_ */
+
 /* vi: set ts=4 sw=4 cindent: */
