@@ -38,7 +38,7 @@ struct sha1;
 
 void entropy_collect(struct sha1 *digest);
 void entropy_minimal_collect(struct sha1 *digest);
-unsigned entropy_random(void);
+uint32 entropy_random(void);
 void entropy_fill(void *buffer, size_t len);
 void entropy_delay(void);
 
