@@ -55,7 +55,8 @@
 
 #endif	/* HAS_STATFS */
 
-#include "lib/fs_free_space.h"
+#include "fs_free_space.h"
+
 #include "override.h"			/* Must be the last header included */
 
 struct fs_info {
@@ -84,7 +85,7 @@ get_fs_info(const char *path, struct fs_info *fsi)
 		struct statvfs buf;
 
 		if (-1 == statvfs(path, &buf)) {
-			g_warning("statvfs(\"%s\") failed: %s", path, g_strerror(errno));
+			g_warning("statvfs(\"%s\") failed: %m", path);
 		} else {
 			free_space = ((filesize_t) 0 + buf.f_bavail) * buf.f_bsize;
 			total_space = ((filesize_t) 0 + buf.f_blocks) * buf.f_frsize;
@@ -96,7 +97,7 @@ get_fs_info(const char *path, struct fs_info *fsi)
 		struct statfs buf;
 
 		if (-1 == statfs(path, &buf)) {
-			g_warning("statfs(\"%s\") failed: %s", path, g_strerror(errno));
+			g_warning("statfs(\"%s\") failed: %m", path);
 		} else {
 			free_space = ((filesize_t) 0 + buf.f_bavail) * buf.f_bsize;
 			total_space = ((filesize_t) 0 + buf.f_blocks) * buf.f_bsize;
@@ -107,7 +108,7 @@ get_fs_info(const char *path, struct fs_info *fsi)
 		struct mingw_statvfs buf;
 
 		if (-1 == mingw_statvfs(path, &buf)) {
-			g_warning("statvfs(\"%s\") failed: %s", path, g_strerror(errno));
+			g_warning("statvfs(\"%s\") failed: %m", path);
 		} else {
 			free_space = ((filesize_t) 0 + buf.f_cavail) * buf.f_csize;
 			total_space = ((filesize_t) 0 + buf.f_clusters) * buf.f_csize;
