@@ -582,9 +582,9 @@ log_fprint(enum log_file which, const struct tm *ct, GLogLevelFlags level,
  * @return pointer to static string.
  */
 const char *
-log_prefix(GLogLevelFlags loglvl)
+log_prefix(GLogLevelFlags level)
 {
-	switch (loglvl) {
+	switch (level & G_LOG_LEVEL_MASK) {
 	case G_LOG_LEVEL_CRITICAL: return "CRITICAL";
 	case G_LOG_LEVEL_ERROR:    return "ERROR";
 	case G_LOG_LEVEL_WARNING:  return "WARNING"; 
@@ -681,8 +681,8 @@ s_minilogv(GLogLevelFlags level, bool copy, const char *fmt, va_list args)
 	DECLARE_STR(11);
 	char time_buf[18];
 	const char *prefix;
-	GLogLevelFlags loglvl;
 	unsigned stid;
+	int saved_errno;
 
 	if G_UNLIKELY(logfile[LOG_STDERR].disabled)
 		return;
@@ -701,8 +701,9 @@ s_minilogv(GLogLevelFlags level, bool copy, const char *fmt, va_list args)
 	if (!copy && !log_printable(LOG_STDERR))
 		return;
 
-	loglvl = level & ~(G_LOG_FLAG_RECURSION | G_LOG_FLAG_FATAL);
-	prefix = log_prefix(loglvl);
+	saved_errno = errno;
+
+	prefix = log_prefix(level);
 	stid = thread_small_id();
 
 	/*
@@ -733,6 +734,8 @@ s_minilogv(GLogLevelFlags level, bool copy, const char *fmt, va_list args)
 	log_flush_err();
 	if (copy && log_stdout_is_distinct())
 		log_flush_out();
+
+	errno = saved_errno;
 }
 
 /**
@@ -783,7 +786,6 @@ s_logv(logthread_t *lt, GLogLevelFlags level, const char *format, va_list args)
 	ckhunk_t *ck;
 	void *saved;
 	bool recursing;
-	GLogLevelFlags loglvl;
 	unsigned stid;
 
 	if (G_UNLIKELY(logfile[LOG_STDERR].disabled))
@@ -803,6 +805,7 @@ s_logv(logthread_t *lt, GLogLevelFlags level, const char *format, va_list args)
 		DECLARE_STR(6);
 		char time_buf[18];
 		const char *caller;
+		bool copy;
 
 		caller = stacktrace_caller_name(2);	/* Could log, so pre-compute */
 
@@ -819,15 +822,15 @@ s_logv(logthread_t *lt, GLogLevelFlags level, const char *format, va_list args)
 		 * A recursion with an error message is always fatal.
 		 */
 
-		if (G_LOG_LEVEL_ERROR == level)
+		if (G_LOG_LEVEL_ERROR & level)
 			log_abort();
 
 		/*
 		 * Use minimal logging.
 		 */
 
-		s_minilogv(level | G_LOG_FLAG_RECURSION, level & G_LOG_FLAG_FATAL,
-			format, args);
+		copy = level & (G_LOG_FLAG_FATAL | G_LOG_LEVEL_CRITICAL);
+		s_minilogv(level | G_LOG_FLAG_RECURSION, copy, format, args);
 		goto done;
 	}
 
@@ -880,9 +883,7 @@ s_logv(logthread_t *lt, GLogLevelFlags level, const char *format, va_list args)
 	 */
 
 	str_vprintf(msg, format, args);
-
-	loglvl = level & ~(G_LOG_FLAG_RECURSION | G_LOG_FLAG_FATAL);
-	prefix = log_prefix(loglvl);
+	prefix = log_prefix(level);
 
 	/*
 	 * Avoid stdio's fprintf() from within a signal handler since we
@@ -916,9 +917,8 @@ s_logv(logthread_t *lt, GLogLevelFlags level, const char *format, va_list args)
 		log_flush_err();
 
 		if G_UNLIKELY(
-			(level & G_LOG_FLAG_FATAL) ||
-			G_LOG_LEVEL_CRITICAL == loglvl ||
-			G_LOG_LEVEL_ERROR == loglvl
+			level &
+				(G_LOG_FLAG_FATAL | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_ERROR)
 		) {
 			if (log_stdout_is_distinct())
 				log_flush_out();
@@ -964,7 +964,7 @@ s_logv(logthread_t *lt, GLogLevelFlags level, const char *format, va_list args)
 	 * until the end to avoid recursion.
 	 */
 
-	if G_UNLIKELY(G_LOG_LEVEL_CRITICAL == loglvl || G_LOG_LEVEL_ERROR == loglvl)
+	if G_UNLIKELY(level & (G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_ERROR))
 		s_stacktrace(TRUE, 2);
 
 done:
@@ -1198,10 +1198,13 @@ s_minilog(GLogLevelFlags flags, const char *format, ...)
 void
 s_minierror(const char *format, ...)
 {
+	static int recursion;
 	va_list args;
 	char data[LOG_MSG_MAXLEN];
 	char time_buf[18];
-	DECLARE_STR(4);
+	DECLARE_STR(6);
+
+	recursion++;
 
 	va_start(args, format);
 	str_vbprintf(data, sizeof data, format, args);
@@ -1209,10 +1212,18 @@ s_minierror(const char *format, ...)
 
 	crash_time(time_buf, sizeof time_buf);
 	print_str(time_buf);					/* 0 */
-	print_str(" (CRITICAL): ");				/* 1 */
-	print_str(data);						/* 2 */
-	print_str("\n");						/* 3 */
+	print_str(" (ERROR)");					/* 1 */
+	if (recursion > 1)
+		print_str(" [RECURSIVE]");			/* 2 */
+	print_str(": ");						/* 3 */
+	print_str(data);						/* 4 */
+	print_str("\n");						/* 5 */
 	flush_err_str();
+	if (log_stdout_is_distinct())
+		log_flush_out();
+
+	if (1 == recursion)
+		s_stacktrace(TRUE, 1);
 
 	abort();
 }
@@ -1226,11 +1237,14 @@ s_minierror(const char *format, ...)
 void
 s_minicrit(const char *format, ...)
 {
+	bool in_signal_handler = signal_in_handler();
 	va_list args;
 
 	va_start(args, format);
-	s_minilogv(G_LOG_LEVEL_CRITICAL, FALSE, format, args);
+	s_minilogv(G_LOG_LEVEL_CRITICAL, TRUE, format, args);
 	va_end(args);
+
+	s_stacktrace(in_signal_handler, 1);
 }
 
 /**
@@ -1654,7 +1668,6 @@ log_handler(const char *domain, GLogLevelFlags level,
 	struct tm *ct;
 	const char *prefix;
 	char *safer;
-	GLogLevelFlags loglvl;
 	unsigned stid;
 
 	(void) unused_data;
@@ -1665,8 +1678,7 @@ log_handler(const char *domain, GLogLevelFlags level,
 	now = tm_time_exact();
 	ct = localtime(&now);
 
-	loglvl = level & ~(G_LOG_FLAG_RECURSION | G_LOG_FLAG_FATAL);
-	prefix = log_prefix(loglvl);
+	prefix = log_prefix(level);
 	stid = thread_small_id();
 
 	if (level & G_LOG_FLAG_RECURSION) {
@@ -1679,9 +1691,8 @@ log_handler(const char *domain, GLogLevelFlags level,
 	log_fprint(LOG_STDERR, ct, level, prefix, stid, safer);
 
 	if G_UNLIKELY(
-		(level & G_LOG_FLAG_FATAL) ||
-		G_LOG_LEVEL_CRITICAL == loglvl ||
-		G_LOG_LEVEL_ERROR == loglvl
+		level &
+			(G_LOG_FLAG_FATAL | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_ERROR)
 	) {
 		if (log_stdout_is_distinct())
 			log_fprint(LOG_STDOUT, ct, level, prefix, stid, safer);
@@ -1690,9 +1701,10 @@ log_handler(const char *domain, GLogLevelFlags level,
 	}
 
 	if G_UNLIKELY(
-		G_LOG_LEVEL_CRITICAL == loglvl ||
-		G_LOG_LEVEL_ERROR == loglvl ||
-		(level & (G_LOG_FLAG_RECURSION|G_LOG_FLAG_FATAL))
+		level & (
+			G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION |
+			G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_ERROR
+		)
 	) {
 		stacktrace_where_sym_print_offset(stderr, 3);
 		if (log_stdout_is_distinct()) {
