@@ -43,7 +43,6 @@
 #include "atoms.h"		/* For binary_hash() */
 #include "cq.h"
 #include "endian.h"		/* For peek_*() and poke_*() */
-#include "glib-missing.h"
 #include "hashing.h"
 #include "hashtable.h"
 #include "leak.h"
@@ -52,10 +51,11 @@
 #include "parse.h"		/* For parse_pointer() */
 #include "path.h"		/* For filepath_basename() */
 #include "stacktrace.h"
+#include "str.h"
 #include "tm.h"			/* For tm_time() */
 #include "unsigned.h"	/* For size_is_non_negative() */
+#include "vsort.h"
 #include "xmalloc.h"
-#include "xsort.h"
 
 /*
  * The following setups are more or less independent from each other.
@@ -1282,7 +1282,7 @@ malloc_log_block(const void *k, void *v, void *leaksort)
 		return;
 
 #ifdef MALLOC_TIME
-	gm_snprintf(ago, sizeof ago, " [%s]",
+	str_bprintf(ago, sizeof ago, " [%s]",
 		short_time(delta_time(tm_time(), b->ttime)));
 #else
 	ago[0] = '\0';
@@ -1375,7 +1375,7 @@ malloc_log_real_block(const void *k, void *v, void *leaksort)
 		return;		/* Was already logged through malloc_log_block() */
 
 #ifdef MALLOC_TIME
-	gm_snprintf(ago, sizeof ago, " [%s]",
+	str_bprintf(ago, sizeof ago, " [%s]",
 		short_time(delta_time(tm_time(), rb->atime)));
 #else
 	ago[0] = '\0';
@@ -2792,7 +2792,7 @@ alloc_dump(FILE *f, bool total)
 	 */
 
 	hash_table_foreach(stats, stats_fill_array, &filler);
-	qsort(filler.stats, count, sizeof(struct stats *),
+	vsort(filler.stats, count, sizeof(struct stats *),
 		total ? stats_total_allocated_cmp : stats_allocated_cmp);
 
 	/*
@@ -2811,7 +2811,7 @@ alloc_dump(FILE *f, bool total)
 	filler.idx = 0;
 
 	hash_table_foreach(stats, stats_fill_array, &filler);
-	qsort(filler.stats, count, sizeof(struct stats *),
+	vsort(filler.stats, count, sizeof(struct stats *),
 		total ? stats_total_residual_cmp : stats_residual_cmp);
 
 	fprintf(f, "--- summary by decreasing %s residual memory size %s %s:\n",
@@ -2828,7 +2828,7 @@ alloc_dump(FILE *f, bool total)
 		filler.idx = 0;
 
 		hash_table_foreach(stats, stats_fill_array, &filler);
-		xqsort(filler.stats, count, sizeof(struct stats *),
+		vsort(filler.stats, count, sizeof(struct stats *),
 			stats_total_residual_cmp);
 
 		fprintf(f, "--- summary by decreasing %s residual memory size %s %s:\n",
