@@ -51,6 +51,7 @@
 #include "stacktrace.h"
 #include "stringify.h"
 #include "unsigned.h"
+#include "vsort.h"
 #include "xmalloc.h"
 
 #include "override.h"			/* Must be the last header included */
@@ -588,10 +589,10 @@ memusage_sort_frames(const memusage_t *mu, bool periodic,
 	 */
 
 	count = hash_table_size(ht);
-	fill->array = xmalloc(count * sizeof fill->array[0]);
+	XMALLOC_ARRAY(fill->array, count);
 	while (hash_table_size(ht) != count) {
 		count = hash_table_size(ht);
-		fill->array = xrealloc(fill->array, count * sizeof fill->array[0]);
+		XREALLOC_ARRAY(fill->array, count);
 	}
 	fill->capacity = count;
 	fill->count = 0;
@@ -600,7 +601,7 @@ memusage_sort_frames(const memusage_t *mu, bool periodic,
 
 	g_assert(fill->count == fill->capacity);
 
-	qsort(fill->array, fill->count, sizeof fill->array[0], callframe_cmp);
+	vsort(fill->array, fill->count, sizeof fill->array[0], callframe_cmp);
 }
 
 /**
