@@ -37,6 +37,12 @@
 void *omalloc(size_t size) WARN_UNUSED_RESULT G_GNUC_MALLOC;
 void *omalloc0(size_t size) WARN_UNUSED_RESULT G_GNUC_MALLOC;
 char *ostrdup(const char *str) WARN_UNUSED_RESULT G_GNUC_MALLOC;
+char *ostrndup(const char *str, size_t n) WARN_UNUSED_RESULT G_GNUC_MALLOC;
+const char *ostrdup_readonly(const char *str) WARN_UNUSED_RESULT G_GNUC_MALLOC;
+const void *ocopy_readonly(const void *p, size_t size)
+	WARN_UNUSED_RESULT G_GNUC_MALLOC;
+const char *ostrndup_readonly(const char *str, size_t n)
+	WARN_UNUSED_RESULT G_GNUC_MALLOC;
 
 static inline void * WARN_UNUSED_RESULT G_GNUC_MALLOC
 ocopy(const void *p, size_t size)
@@ -49,7 +55,7 @@ ocopy(const void *p, size_t size)
 struct logagent;
 
 size_t omalloc_page_count(void);
-void set_omalloc_debug(guint32 level);
+void set_omalloc_debug(uint32 level);
 void omalloc_close(void);
 void omalloc_dump_stats_log(struct logagent *la, unsigned options);
 
