@@ -36,24 +36,11 @@
 
 #include "common.h"
 
-/**
- * Incremental deflater stream.
- */
-typedef struct  {
-	const void *in;		/**< Buffer being compressed */
-	int inlen;			/**< Length of input buffer */
-	void *out;			/**< Compressed data */
-	int outlen;			/**< Length of ouput buffer */
-	int inlen_total;	/**< Total input length seen */
-	void *opaque;		/**< Internal data structures */
-	uint allocated:1;	/**< Is output buffer was allocated or static? */
-	uint closed:1;		/**< Whether the stream was closed */
-} zlib_deflater_t;
+struct zlib_deflater;
+typedef struct zlib_deflater zlib_deflater_t;
 
-#define zlib_deflater_out(z)	((z)->out)
-#define zlib_deflater_outlen(z)	((z)->outlen)
-#define zlib_deflater_inlen(z)	((z)->inlen_total)
-#define zlib_deflater_closed(z)	((z)->closed)
+struct zlib_inflater;
+typedef struct zlib_inflater zlib_inflater_t;
 
 /*
  * Public interface.
@@ -65,9 +52,39 @@ zlib_deflater_t *zlib_deflater_make(const void *data, int len, int level);
 zlib_deflater_t *zlib_deflater_make_into(
 	const void *data, int len, void *dest, int destlen, int level);
 int zlib_deflate(zlib_deflater_t *zd, int amount);
-bool zlib_deflate_data(zlib_deflater_t *zd, const void *data, int len);
+int zlib_deflate_all(zlib_deflater_t *zd);
+int zlib_deflate_step(zlib_deflater_t *zd, int amount, bool may_close);
+bool zlib_deflate_data(zlib_deflater_t *zd,
+	const void *data, int len, bool final);
 bool zlib_deflate_close(zlib_deflater_t *zd);
 void zlib_deflater_free(zlib_deflater_t *zd, bool output);
+void zlib_deflater_reset(zlib_deflater_t *zd, const void *data, int len);
+void zlib_deflater_reset_into(zlib_deflater_t *zd,
+	const void *data, int len, void *dest, int destlen);
+
+bool zlib_deflater_closed(const struct zlib_deflater *zd);
+int zlib_deflater_inlen(const struct zlib_deflater *zd);
+int zlib_deflater_outlen(const struct zlib_deflater *zd);
+void *zlib_deflater_out(const struct zlib_deflater *zd);
+
+zlib_inflater_t *zlib_inflater_make(const void *data, int len);
+zlib_inflater_t *zlib_inflater_make_into(
+	const void *data, int len, void *dest, int destlen);
+int zlib_inflate(zlib_inflater_t *zi, int amount);
+int zlib_inflate_step(zlib_inflater_t *zi, int amount, bool may_close);
+int zlib_inflate_data(zlib_inflater_t *zi, const void *data, int len);
+bool zlib_inflate_close(zlib_inflater_t *zi);
+void zlib_inflater_free(zlib_inflater_t *zi, bool output);
+void zlib_inflater_reset(zlib_inflater_t *zi, const void *data, int len);
+void zlib_inflater_reset_into(zlib_inflater_t *zi,
+	const void *data, int len, void *dest, int destlen);
+
+bool zlib_inflater_closed(const struct zlib_inflater *zi);
+int zlib_inflater_inlen(const struct zlib_inflater *zi);
+int zlib_inflater_outlen(const struct zlib_inflater *zi);
+void *zlib_inflater_out(const struct zlib_inflater *zi);
+size_t zlib_inflater_maxoutlen(const struct zlib_inflater *zi);
+void zlib_inflater_set_maxoutlen(struct zlib_inflater *zi, size_t len);
 
 void *zlib_uncompress(const void *data, int len, ulong uncompressed_len);
 int zlib_inflate_into(const void *data, int len, void *out, int *outlen);
