@@ -50,7 +50,7 @@
 
 #include "lib/concat.h"
 #include "lib/host_addr.h"
-#include "lib/glib-missing.h"
+#include "lib/str.h"
 #include "lib/stringify.h"
 #include "lib/timestamp.h"
 #include "lib/tm.h"
@@ -175,7 +175,7 @@ shell_exec_status(struct gnutella_shell *sh, int argc, const char *argv[])
 			fd = empty;
 		}
 
-		gm_snprintf(flags, sizeof flags,
+		str_bprintf(flags, sizeof flags,
 			"<%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s>",
 			pmp,
 			GNET_PROPERTY(download_queue_frozen) ? "DFZ " : empty,
@@ -195,7 +195,7 @@ shell_exec_status(struct gnutella_shell *sh, int argc, const char *argv[])
 			fw, dht,
 			settings_is_ultra() ? "UP" : "LF");
 
-		gm_snprintf(buf, sizeof buf,
+		str_bprintf(buf, sizeof buf,
 			"+%s+\n"
 			"| %-18s%51s |\n"
 			"|%s|\n",
@@ -225,7 +225,7 @@ shell_exec_status(struct gnutella_shell *sh, int argc, const char *argv[])
 			blackout = "None";
 		}
 
-		gm_snprintf(buf, sizeof buf,
+		str_bprintf(buf, sizeof buf,
 			"|   Mode: %-9s                   Last Switch: %-19s%2s|\n"
 			"| Uptime: %-9s                    Last Check: %-19s%2s|\n"
 			"|   Port: %-9u                      Blackout: %-7s%14s|\n"
@@ -247,7 +247,7 @@ shell_exec_status(struct gnutella_shell *sh, int argc, const char *argv[])
 	switch (GNET_PROPERTY(network_protocol)) {
 	case NET_USE_BOTH:
 	case NET_USE_IPV4:
-		gm_snprintf(buf, sizeof buf,
+		str_bprintf(buf, sizeof buf,
 			"| IPv4: %-44s Since: %-12s|\n",
 			host_addr_to_string(listen_addr()),
 			short_time(delta_time(now, GNET_PROPERTY(current_ip_stamp))));
@@ -257,11 +257,11 @@ shell_exec_status(struct gnutella_shell *sh, int argc, const char *argv[])
 	/* IPv6 info */ 
 	switch (GNET_PROPERTY(network_protocol)) {
 	case NET_USE_BOTH:
-		gm_snprintf(buf, sizeof buf, "|%s|\n", dashes);
+		str_bprintf(buf, sizeof buf, "|%s|\n", dashes);
 		shell_write(sh, buf);
 		/* FALL THROUGH */
 	case NET_USE_IPV6:
-		gm_snprintf(buf, sizeof buf,
+		str_bprintf(buf, sizeof buf,
 			"| IPv6: %-44s Since: %-12s|\n",
 			host_addr_to_string(listen_addr6()),
 			short_time(delta_time(now, GNET_PROPERTY(current_ip6_stamp))));
@@ -269,7 +269,7 @@ shell_exec_status(struct gnutella_shell *sh, int argc, const char *argv[])
 	}
 
 	/* Node counts */
-	gm_snprintf(buf, sizeof buf,
+	str_bprintf(buf, sizeof buf,
 		"|%s|\n"
 		"| Peers: %-7u Ultra %4u/%-7u  Leaf %4u/%-6u  Legacy %4u/%-4u |\n"
 		"|            Downloads %4u/%-4u  Uploads %4u/%-7u Browse %4u/%-4u |\n"
@@ -336,7 +336,7 @@ shell_exec_status(struct gnutella_shell *sh, int argc, const char *argv[])
 		dht_out = short_rate_get_string(
 			cur ? bw_stats.current : bw_stats.average, metric);
 
-		gm_snprintf(buf, sizeof buf,
+		str_bprintf(buf, sizeof buf,
 			"| %-70s|\n"
 			"|%71s|\n"
 			"| %5s  In:  %13s %13s %13s %13s   |\n"
@@ -353,7 +353,7 @@ shell_exec_status(struct gnutella_shell *sh, int argc, const char *argv[])
 		char line[128];
 		bool metric = GNET_PROPERTY(display_metric_units);
 
-		gm_snprintf(buf, sizeof buf, "|%s|\n", equals);
+		str_bprintf(buf, sizeof buf, "|%s|\n", equals);
 		shell_write(sh, buf);
 		concat_strings(line, sizeof line,
 			"Shares ",
@@ -364,13 +364,13 @@ shell_exec_status(struct gnutella_shell *sh, int argc, const char *argv[])
 			short_kb_size(shared_kbytes_scanned(), metric),
 			" total",
 			(void *) 0);
-		gm_snprintf(buf, sizeof buf,
+		str_bprintf(buf, sizeof buf,
 			"| %-35s Up: %-11s Down: %-11s |\n",
 			line,
 			short_byte_size(GNET_PROPERTY(ul_byte_count), metric),
 			short_byte_size2(GNET_PROPERTY(dl_byte_count), metric));
 		shell_write(sh, buf);
-		gm_snprintf(buf, sizeof buf, "+%s+\n", dashes);
+		str_bprintf(buf, sizeof buf, "+%s+\n", dashes);
 		shell_write(sh, buf);
 	}
 
