@@ -1,10 +1,6 @@
 /*
- * $Id: drop.h,v 1.1 2004/08/24 21:45:27 rmanfredi Exp $
- *
  * Copyright (c) 2004, Christian Biere
  *
- * Drop support - no dragging, just dropping.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,11 +21,28 @@
  *----------------------------------------------------------------------
  */
 
-#ifndef _drop_init_h_
-#define _drop_init_h_
+/**
+ * @ingroup gtk
+ * @file
+ *
+ * Drop support - no dragging, just dropping.
+ *
+ * @author Christian Biere
+ * @date 2004
+ */
 
-void drop_init(void);
-void drop_close(void);
+#ifndef _drop_h_
+#define _drop_h_
+
+#include "gui.h"
+
+typedef void (*drag_data_received_cb)(GtkWidget *widget, GdkDragContext *dc,
+				gint x, gint y, GtkSelectionData *data,
+				guint info, guint stamp, gpointer udata);
+
+void drop_widget_init(GtkWidget *widget, drag_data_received_cb callback,
+		void *user_data);
+
+#endif /* _drop_h_ */
 
-#endif /* _drop_init_h_ */
 /* vi: set ts=4 sw=4 cindent: */
