@@ -291,6 +291,8 @@ void
 fi_gui_clear_sources(void)
 {
    	gtk_clist_clear(clist_download_sources);
+	htable_clear(fi_sources);
+	htable_clear(source_rows);
 }
 
 
@@ -334,16 +336,57 @@ on_clist_download_sources_row_moved(int dst, void *user_data)
 static void
 on_clist_download_sources_row_removed(void *data)
 {
+	int count;
+
 	download_check(data);
+	g_return_unless(htable_contains(fi_sources, data));
+	g_soft_assert_log(htable_count(fi_sources) == htable_count(source_rows),
+		"fi_sources count: %zu, source_rows count: %zu",
+		htable_count(fi_sources), htable_count(source_rows));
+
+	count = htable_count(fi_sources);		/* Old row count */
 	htable_remove(fi_sources, data);
+
 	clist_sync_rows(clist_download_sources,
 		on_clist_download_sources_row_moved);
+
+	htable_remove(source_rows, int_to_pointer(count - 1));	/* Last row gone */
+
+	g_soft_assert_log(htable_count(fi_sources) == htable_count(source_rows),
+		"fi_sources count: %zu, source_rows count: %zu",
+		htable_count(fi_sources), htable_count(source_rows));
+}
+
+/**
+ * Called before invoking fi_gui_source_show() repeatedly.
+ *
+ * This allows freezing the download clist to avoid a costly redrawing of
+ * the list after each addition, preventing O(n^2) GUI updates.
+ */
+void
+fi_gui_source_massive_update(bool starting)
+{
+	GtkCList *clist = clist_download_sources;
+
+	g_return_if_fail(clist);
+
+	/*
+	 * Freeze / thaw operations are counted by the clist, so it is perfectly
+	 * fine to freeze here even though fi_gui_source_show() will also perform
+	 * some freeze/thaw operations.
+	 */
+
+	if (starting) {
+		gtk_clist_freeze(clist);
+	} else {
+		gtk_clist_thaw(clist);
+	}
 }
 
 void
 fi_gui_source_show(struct download *key)
 {
-	const char *titles[c_fi_sources];
+	const char *titles[c_src_num];
 	GtkCList *clist;
 	unsigned i;
 	int row;
@@ -355,16 +398,24 @@ fi_gui_source_show(struct download *key)
 	for (i = 0; i < G_N_ELEMENTS(titles); i++) {
 		titles[i] = "";
 	}
+
+	gtk_clist_freeze(clist);
 	row = gtk_clist_append(clist, (char **) titles);
 	g_return_if_fail(row >= 0);
 
 	htable_insert(fi_sources, key, int_to_pointer(row));
 	htable_insert(source_rows, int_to_pointer(row), key);
+
+	g_soft_assert_log(htable_count(fi_sources) == htable_count(source_rows),
+		"fi_sources count: %zu, source_rows count: %zu",
+		htable_count(fi_sources), htable_count(source_rows));
+
 	gtk_clist_set_row_data_full(clist, row, key,
 		on_clist_download_sources_row_removed);
-	for (i = 0; i < c_fi_sources; i++) {
+	for (i = 0; i < G_N_ELEMENTS(titles); i++) {
 		render_sources(key, row, i);
 	}
+	gtk_clist_thaw(clist);
 }
 
 void
@@ -455,7 +506,7 @@ fi_gui_source_update(struct download *d)
 	if (htable_lookup_extended(fi_sources, d, NULL, &value)) {
 		int i, row = pointer_to_int(value);
 
-		for (i = 0; i < c_fi_sources; i++) {
+		for (i = 0; i < c_src_num; i++) {
 			render_sources(d, row, i);
 		}
 	}
