@@ -66,7 +66,8 @@
  * This widget is included in GTK+ since 2.5.0
  */
 
-#include "lib/glib-missing.h"
+#include "lib/str.h"
+
 #include "lib/override.h"		/* Must be the last header included */
 
 static void gtk_cell_renderer_progress_init(GtkCellRendererProgress *);
@@ -383,11 +384,11 @@ gtk_cell_renderer_progress_render(
 	gdk_draw_rectangle(window, gc, TRUE, x + 2, y + 2, perc_w, h - 4);
 
 	val = cellprogress->priv->value;
-	gm_snprintf(text, sizeof text, "%d", val);
+	str_bprintf(text, sizeof text, "%d", val);
 	layout = gtk_widget_create_pango_layout(widget, text);
 	pango_layout_get_pixel_extents(layout, NULL, &logical_rect);
 	g_object_unref(G_OBJECT (layout));
-	gm_snprintf(text, sizeof text, "%d %%", val);
+	str_bprintf(text, sizeof text, "%d %%", val);
 	layout = gtk_widget_create_pango_layout(widget, text);
 
 	pos = (w - logical_rect.width) / 2;
