@@ -1,6 +1,4 @@
 /*
- * $Id: uploads.h,v 1.10 2005/12/11 13:52:51 rmanfredi Exp $
- *
  * Copyright (c) 2001-2003, Richard Eckart
  *
  *----------------------------------------------------------------------
@@ -43,18 +41,16 @@ typedef struct upload_row_data {
     GtkTreeIter   iter;
 	host_addr_t   addr;
 	filesize_t    size;
+	const gchar   *user_agent;	/**< atom */
+	const gchar   *name;		/**< atom */
 	gboolean      push;
-	gchar         *user_agent;	/**< atom */
-	gchar         *name;		/**< atom */
-	gint          country;
+	guint16       country;
 #endif /* USE_GTK2 */
 } upload_row_data_t;
 
-void uploads_gui_early_init(void);
 void uploads_gui_init(void);
 void uploads_gui_shutdown(void);
 
-void uploads_gui_update_display(time_t now);
 void uploads_gui_clear_completed(void);
 
 #endif /* _gtk_uploads_h_ */
