@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010, Raphael Manfredi
+ * Copyright (c) 2010, 2012 Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -28,7 +28,7 @@
  * UPnP service control.
  *
  * @author Raphael Manfredi
- * @date 2010
+ * @date 2010, 2012
  */
 
 #include "common.h"
@@ -46,7 +46,6 @@
 #include "xml/xfmt.h"
 
 #include "lib/atoms.h"
-#include "lib/glib-missing.h"
 #include "lib/nv.h"
 #include "lib/parse.h"
 #include "lib/str.h"
@@ -594,7 +593,7 @@ upnp_ctrl_launch(const upnp_service_t *usd, const char *action,
 	{
 		char ns[256];
 		
-		gm_snprintf(ns, sizeof ns, "%s%s:%u",
+		str_bprintf(ns, sizeof ns, "%s%s:%u",
 			UPNP_NS_BASE,
 			upnp_service_type_to_string(upnp_service_type(usd)),
 			upnp_service_version(usd));
@@ -643,7 +642,7 @@ upnp_ctrl_launch(const upnp_service_t *usd, const char *action,
 		if (host_addr_net(upnp_get_local_addr()) == NET_TYPE_NONE)
 			options |= SOAP_RPC_O_LOCAL_ADDR;
 
-		gm_snprintf(action_uri, sizeof action_uri, "%s#%s",
+		str_bprintf(action_uri, sizeof action_uri, "%s#%s",
 			xnode_element_ns(root), action);
 
 		ucd->action = atom_str_get(action_uri);
@@ -1096,6 +1095,54 @@ upnp_ctrl_DeletePortMapping(const upnp_service_t *usd,
 		NULL);
 }
 
+/**
+ * Process returned value from GetStatusInfo().
+ *
+ * @return walloc()'ed structure (whose size is written in lenp) containing
+ * the decompiled arguments, or NULL if the returned arguments cannot be
+ * processed.
+ */
+static void *
+upnp_ctrl_ret_GetStatusInfo(nv_table_t *ret, size_t *lenp)
+{
+	struct upnp_GetStatusInfo *r;
+	const char *status;
+	time_delta_t uptime;
+
+	status = nv_table_lookup_str(ret, "NewConnectionStatus");
+	if (NULL == status)
+		return NULL;
+
+	if (!upnp_ctrl_get_time_delta(ret, "NewUptime", &uptime))
+		return NULL;
+
+	WALLOC(r);
+	r->connection_status = status;
+	r->uptime = uptime;
+	*lenp = sizeof *r;
+
+	return r;
+}
+
+/**
+ * Get status information [IP or PPP connection].
+ *
+ * @param usd		the UPnP service to contact
+ * @param cb		callback to invoke when reply is available
+ * @param arg		additional callback argument
+ *
+ * @return UPnP request handle if the SOAP RPC was initiated, NULL otherwise
+ * (in which case callbacks will never be called).
+ */
+upnp_ctrl_t *
+upnp_ctrl_GetStatusInfo(const upnp_service_t *usd,
+	upnp_ctrl_cb_t cb, void *arg)
+{
+	return upnp_ctrl_launch(usd, "GetStatusInfo",
+		NULL, 0, cb, arg,
+		upnp_ctrl_ret_GetStatusInfo);
+}
+
 /**
  * Process returned value from GetTotalPacketsReceived().
  *
